#ifndef _RTP_SOCKET_HH_
#define _RTP_SOCKET_HH_

#include <errno.h>
#include "sys_time.hh"
#include "signal_handler.hh"
#include "socket.hh"
#include "alarm.hh"
#include "message.hh"
#include "rtp_packet.hh"
#include "rtp_delay.hh"
#include "rtp_socket_ssrc.hh"
#include "list.hh"

class RTP_Socket_IO_Handler;
class RTP_Socket_ALRM_Handler;

extern Alarm_Manager  The_Alarm_Manager;

class RTP_Socket
{
  friend RTP_Socket_IO_Handler;
  friend RTP_Socket_ALRM_Handler;
private:
#ifdef WANT_STATS
  _s_Socket*              psocket;
#else
  Socket*                 psocket;
#endif
  Socket*                 prtcp_socket;
  RTP_Socket_IO_Handler*  prtp_s_io_h;
  RTP_Socket_IO_Handler*  prtcp_s_io_h;
  RTP_Socket_ALRM_Handler*  prtp_s_alrm_h;
  SSRC_Manager            ssrc_manager;
  List<SYS_Address>       members_list;
  int                     asyncmode;
  int                     client_pid;
public:
  RTP_Socket(Port&,int,int);
  ~RTP_Socket();
  virtual void NewPacketFromNet(Message*);
  virtual void NewPlayablePacket(SSRC&);
  int AddMember(UDP_Address&);
  void SetAsync(int);
  int Send(Message&,unsigned int);
  int Receive(void*,unsigned int,SSRC*,unsigned int);
  //unsigned int GetSSRCCount();
  //unsigned int GetSSRCList(SSRC*,unsigned int);
  Socket* Wich_socket();
  int AccessMIB(int*,int,char*);
};

class _s_RTP_Socket : public RTP_Socket
{
private:
  Statistic*              pstat_net;
  Statistic*              pstat_alarm;
public:
  _s_RTP_Socket(Port&,int,int);
  ~_s_RTP_Socket();
  void NewPacketFromNet(Message*);
  void NewPlayablePacket(SSRC&);
};

class RTP_Socket_Signal_Handler : public Signal_Handler
{
protected: 
  RTP_Socket*   pOwner_rtp_socket;
public:
  RTP_Socket_Signal_Handler(int,RTP_Socket*);
  ~RTP_Socket_Signal_Handler();
};

class RTP_Socket_IO_Handler : public RTP_Socket_Signal_Handler
{
public:
  RTP_Socket_IO_Handler(RTP_Socket*);
  ~RTP_Socket_IO_Handler();
  void Handler();
};

class RTP_Socket_ALRM_Handler : public RTP_Socket_Signal_Handler
{
public:
  RTP_Socket_ALRM_Handler(RTP_Socket*);
  ~RTP_Socket_ALRM_Handler();
  void Handler();
};


#endif
